using System;
using System.Diagnostics;
using System.Windows.Controls;
using System.Windows.Input;

using VS2010Addin.SQLAzure2010Addin.Data;
using VS2010Addin.SQLAzure2010Addin.Properties;
using VS2010Addin.SQLAzure2010Addin.ScriptParsing;
using VS2010Addin.SQLAzure2010Addin.ToolWindows;

namespace VS2010Addin.SQLAzure2010Addin.Commands
{
    public class BaseCommandHandler
    {
        protected readonly ExplorerToolWindow _parentWindow;

        public BaseCommandHandler(ExplorerToolWindow parent)
        {
            _parentWindow = parent;
        }

        public void ScriptAsCreate(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                var menuInfo = menuItem.CommandParameter as MenuCommandParameters;
                var repository = new Repository(Settings.Default.ServerName, Settings.Default.Login, Settings.Default.Password);
                if (menuInfo != null)
                {
                    string script;
                    if(menuInfo.MenuItemType == MenuCommandParameters.MenuType.Table)
                        script = repository.GetTableDefinition(menuInfo.Database, menuInfo.Name);
                    else
                        script = repository.GetObjectDefinition(menuInfo.Database, menuInfo.Name);

                    OpenSqlEditorToolWindow(menuInfo, script);
                }
            }
        }

        private void OpenSqlEditorToolWindow(MenuCommandParameters menuInfo, string script)
        {
            var pkg = _parentWindow.Package as SQLAzure2010AddinPackage;
            Debug.Assert(pkg != null, "Package property of the Explorere Tool Window should never be null, have you tried to create it manually and not through FindToolWindow()?");

            var sqlEditorWindow = pkg.CreateWindow<SqlEditorWindow>(Math.Abs(menuInfo.Database.GetHashCode()));
            var control = sqlEditorWindow.Content as SqlEditorControl;
            control.Database = menuInfo.Database;
            sqlEditorWindow.Caption = menuInfo.Database;
            var pretty = new Parser().Parse(script);
            control.SqlTextBox.Text = pretty;
            return;
        }
    }
}