using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using VS2010Addin.SQLAzure2010Addin.Data;
using VS2010Addin.SQLAzure2010Addin.Dialogs;
using VS2010Addin.SQLAzure2010Addin.Properties;
using VS2010Addin.SQLAzure2010Addin.ToolWindows;
using VS2010Addin.SQLAzure2010Addin.ScriptParsing;

namespace VS2010Addin.SQLAzure2010Addin.Commands
{
    public class CreateNewMenuCommandsHandler 
    {
        private readonly ExplorerToolWindow _parentWindow;

        public CreateNewMenuCommandsHandler(ExplorerToolWindow parent)
        {
            _parentWindow = parent;
        }

        public void CreateNewTable(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;

            var info = menuItem.CommandParameter as CreateNewCommandParameters;
            if (info == null) return;

            var repository = new Repository(Settings.Default.ServerName, Settings.Default.Login, Settings.Default.Password);
            string script = repository.GetCreateScript("new_table");

            OpenSqlEditorToolWindow(info, script);
        }

        public void CreateNewView(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;

            var info = menuItem.CommandParameter as CreateNewCommandParameters;
            if (info == null) return;

            var repository = new Repository(Settings.Default.ServerName, Settings.Default.Login, Settings.Default.Password);
            string script = repository.GetCreateScript("new_view");

            OpenSqlEditorToolWindow(info, script);
        }

        public void CreateNewSP(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;

            var info = menuItem.CommandParameter as CreateNewCommandParameters;
            if (info == null) return;

            var repository = new Repository(Settings.Default.ServerName, Settings.Default.Login, Settings.Default.Password);
            string script = repository.GetCreateScript("new_sp");

            OpenSqlEditorToolWindow(info, script);
        }

        public void CreateNewTableFunction(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;

            var info = menuItem.CommandParameter as CreateNewCommandParameters;
            if (info == null) return;

            var repository = new Repository(Settings.Default.ServerName, Settings.Default.Login, Settings.Default.Password);
            string script = repository.GetCreateScript("new_table_function");

            OpenSqlEditorToolWindow(info, script);
        }

        public void CreateNewScalarFunction(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;

            var info = menuItem.CommandParameter as CreateNewCommandParameters;
            if (info == null) return;

            var repository = new Repository(Settings.Default.ServerName, Settings.Default.Login, Settings.Default.Password);
            string script = repository.GetCreateScript("new_scalar_function");

            OpenSqlEditorToolWindow(info, script);
        }


        private void OpenSqlEditorToolWindow(BaseCommandParameters menuInfo, string script)
        {
            var pkg = _parentWindow.Package as SQLAzure2010AddinPackage;
            Debug.Assert(pkg != null, "Package property of the Explorere Tool Window should never be null, have you tried to create it manually and not through FindToolWindow()?");

            var sqlEditorWindow = pkg.CreateWindow<SqlEditorWindow>(Math.Abs(menuInfo.Database.GetHashCode()));
            var control = sqlEditorWindow.Content as SqlEditorControl;
            control.Database = menuInfo.Database;
            sqlEditorWindow.Caption = menuInfo.Database;
            //var pretty = new Parser().Parse(script);
            control.SqlTextBox.Text = script;
            control.SqlTextBox.Focus();
            return;
        }
    }
}