﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

using VS2010Addin.SQLAzure2010Addin.Data;
using VS2010Addin.SQLAzure2010Addin.Helpers;
using VS2010Addin.SQLAzure2010Addin.Properties;
using VS2010Addin.SQLAzure2010Addin.ToolWindows;

namespace VS2010Addin.SQLAzure2010Addin.Commands
{
    public class DatabaseMenuCommandsHandler
    {
        private readonly ExplorerToolWindow _parentWindow;

        public DatabaseMenuCommandsHandler(ExplorerToolWindow parent)
        {
            _parentWindow = parent;
        }

        private ExplorerControl _explorerControl;

        public void CreateDatabase(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;

            var databaseInfo = menuItem.CommandParameter as DatabasesMenuCommandParameters;
            if (databaseInfo == null) return;

            _explorerControl = databaseInfo.ExplorerControl;
            var treeViewItem = databaseInfo.DatabasesTreeViewItem;

            var panel = new StackPanel { Width = 90, Orientation = Orientation.Horizontal };
            var image = ImageHelper.GetImageFromResource("../resources/database.png");
            image.Width = 20;
            image.Height = 20;
            var databaseTextBox = new TextBox { Width = 70 };
            databaseTextBox.PreviewKeyUp += NewDatabaseEnter;
            databaseTextBox.Loaded += (sender1, e1) => ((TextBox)sender1).Focus();
            databaseTextBox.LostFocus += (sender2, e2) => treeViewItem.Items.Remove(panel);

            panel.Children.Add(image);
            panel.Children.Add(databaseTextBox);

            treeViewItem.IsExpanded = true;
            treeViewItem.Items.Insert(0, panel);
            //treeViewItem.Items.Add(panel);
        }

        void NewDatabaseEnter(object sender, KeyEventArgs e)
        {
            var text = ((TextBox)sender).Text;

            if (e.Key == Key.Escape) { _explorerControl.BuildDatabaseTree(); return; }
            if (e.Key != Key.Enter) return;

            if (text.Trim().Length == 0) { e.Handled = true; return; }

            var repository = new Repository(Settings.Default.ServerName, Settings.Default.Login,
                                            Settings.Default.Password);
            try
            {
                repository.CreateDatabase(text);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message,
                                "Error creating database", MessageBoxButton.OK,
                                MessageBoxImage.Error);
            }
            _explorerControl.BuildDatabaseTree();
        }

        public void DropDatabase(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;

            var databaseInfo = menuItem.CommandParameter as DatabaseMenuCommandParameters;
            if (databaseInfo == null) return;

            var answer = MessageBox.Show("Are you really sure you want to drop/delete the database " + databaseInfo.Database + "?",
                                         "Drop database " + databaseInfo.Database, MessageBoxButton.YesNo,
                                         MessageBoxImage.Question, MessageBoxResult.No);
            if (answer == MessageBoxResult.No) return;

            var repository = new Repository(Settings.Default.ServerName, Settings.Default.Login,
                                            Settings.Default.Password);
            try
            {
                repository.DropDatabase(databaseInfo.Database);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message,
                                "Error dropping database", MessageBoxButton.OK,
                                MessageBoxImage.Error);
            }
            databaseInfo.ExplorerControl.BuildDatabaseTree();
        }

        public void SpawnSqlEditorWindow(object sender, ExecutedRoutedEventArgs e)
        {

            var menuItem = sender as MenuItem;
            if (menuItem != null)
            {
                var databaseInfo = menuItem.CommandParameter as DatabaseMenuCommandParameters;
                if (databaseInfo != null)
                {
                    var pkg = _parentWindow.Package as SQLAzure2010AddinPackage;
                    Debug.Assert(pkg != null, "Package property of the Explorere Tool Window should never be null, have you tried to create it manually and not through FindToolWindow()?");

                    var sqlEditorWindow = pkg.CreateWindow<SqlEditorWindow>(Math.Abs(databaseInfo.Database.GetHashCode()));
                    var editorControl = sqlEditorWindow.Content as SqlEditorControl;
                    Debug.Assert(editorControl != null);
                    editorControl.Database = databaseInfo.Database;
                    editorControl.SqlTextBox.Text = "";
                    sqlEditorWindow.Caption = databaseInfo.Database;

                    Debug.Assert(editorControl != null, "The SqlEditorWindow *should* have a editorControl with content.");
                }
            }
        }
    }
}
