using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using VS2010Addin.SQLAzure2010Addin.Data;
using VS2010Addin.SQLAzure2010Addin.Dialogs;
using VS2010Addin.SQLAzure2010Addin.Properties;
using VS2010Addin.SQLAzure2010Addin.ToolWindows;

namespace VS2010Addin.SQLAzure2010Addin.Commands
{
    public class UserMenuCommandsHandler
    {
        private readonly ExplorerControl _parentWindow;

        public UserMenuCommandsHandler(ExplorerControl parent)
        {
            _parentWindow = parent;
        }

        public void DropUser(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;
            
            var userInfo = menuItem.CommandParameter as UserCommandParameters;
            if (userInfo == null) return;
            
            var repository = new Repository(Settings.Default.ServerName, Settings.Default.Login, Settings.Default.Password);
            try
            {
                repository.DropUser(userInfo.User.Database, userInfo.User.Name);
            }
            catch (Exception ex)
            {
                MessageBox.Show("Could not drop user: " + ex.Message,
                                "Error dropping user", MessageBoxButton.OK,
                                MessageBoxImage.Error);
            }
            //refresh user tree
            userInfo.TreeViewItem.Items.Clear();
            userInfo.TreeViewItem.Items.Add("Loading...");
            _parentWindow.GetUsersItems(userInfo.TreeViewItem, null, userInfo.Database);
        }

        public void CreateUser(object sender, ExecutedRoutedEventArgs e)
        {
            var menuItem = sender as MenuItem;
            if (menuItem == null) return;

            var userInfo = menuItem.CommandParameter as UsersCommandParameters;
            if (userInfo == null) return;

            var dialog = new CreateUserDialog(userInfo.Database);
            dialog.ShowDialog();

            //refresh user tree
            userInfo.TreeViewItem.Items.Clear();
            userInfo.TreeViewItem.Items.Add("Loading...");
            _parentWindow.GetUsersItems(userInfo.TreeViewItem, null, userInfo.Database);
        }


    }
}