using System.Windows.Controls;
using System.Windows.Input;
using VS2010Addin.SQLAzure2010Addin.Commands;
using VS2010Addin.SQLAzure2010Addin.Helpers;
using VS2010Addin.SQLAzure2010Addin.ToolWindows;

namespace VS2010Addin.SQLAzure2010Addin.ContextMenues
{
    public class DatabaseContextMenu : ContextMenu
    {
        public DatabaseContextMenu(DatabaseMenuCommandParameters databaseMenuCommandParameters, ExplorerToolWindow parent)
        {
            var dcmd = new DatabaseMenuCommandsHandler(parent);
            var showSqlEditorCommandBinding = new CommandBinding(DatabaseMenuCommands.DatabaseCommand,
                                                                dcmd.SpawnSqlEditorWindow);

            var showSqlEditorMenuItem = new MenuItem
            {
                Header = "Open SQL Editor",
                Icon = ImageHelper.GetImageFromResource("../resources/sqlEditor.png"),
                Command = DatabaseMenuCommands.DatabaseCommand,
                CommandParameter = databaseMenuCommandParameters
            };
            showSqlEditorMenuItem.CommandBindings.Add(showSqlEditorCommandBinding);
            Items.Add(showSqlEditorMenuItem);

            var dropDatabaseCommandBinding = new CommandBinding(DatabaseMenuCommands.DatabaseCommand,
                                                                dcmd.DropDatabase);
            var dropDatabaseMenuItem = new MenuItem
            {
                Header = "Drop database",
                Icon = ImageHelper.GetImageFromResource("../resources/delete.png"),
                Command = DatabaseMenuCommands.DatabaseCommand,
                CommandParameter = databaseMenuCommandParameters
            };
            dropDatabaseMenuItem.CommandBindings.Add(dropDatabaseCommandBinding);
            Items.Add(dropDatabaseMenuItem);
        }
    }

    public class TableContextMenu : ContextMenu
    {

        public TableContextMenu(MenuCommandParameters menuCommandParameters, ExplorerToolWindow parent)
        {
            var tcmd = new TableMenuCommandsHandler(parent);
            var showTableDataCommandBinding = new CommandBinding(TableMenuCommands.TableCommand, tcmd.SpawnTableDataWindow);
            var showTableDataMenuItem = new MenuItem
                                            {
                                                Header = "Select top 100 rows",
                                                Icon = ImageHelper.GetImageFromResource("../resources/show_table_data.png"),
                                                Command = TableMenuCommands.TableCommand,
                                                CommandParameter = menuCommandParameters
                                            };
            showTableDataMenuItem.CommandBindings.Add(showTableDataCommandBinding);
            Items.Add(showTableDataMenuItem);

            CreateScriptAsCreateMenuItem(tcmd, menuCommandParameters);
        }

        private void CreateScriptAsCreateMenuItem(TableMenuCommandsHandler tcmd, MenuCommandParameters menuCommandParameters)
        {
            var scriptCommandBinding = new CommandBinding(TableMenuCommands.TableCommand, tcmd.ScriptAsCreate);
            var scriptMenuItem = new MenuItem
                                     {
                                         Header = "Script as CREATE",
                                         Icon = ImageHelper.GetImageFromResource("../resources/sp.png"),
                                         Command = TableMenuCommands.TableCommand,
                                         CommandParameter = menuCommandParameters
                                     };
            scriptMenuItem.CommandBindings.Add(scriptCommandBinding);
            Items.Add(scriptMenuItem);
        }
    }
}