﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;

using VS2010Addin.SQLAzure2010Addin.Data;
using VS2010Addin.SQLAzure2010Addin.Properties;

namespace VS2010Addin.SQLAzure2010Addin.Dialogs
{
    /// <summary>
    /// Interaction logic for CreateLoginDialog.xaml
    /// </summary>
    public partial class CreateUserDialog : Window
    {
        private readonly string _database;
        private readonly Repository _repository;

        public CreateUserDialog(string database)
        {
            InitializeComponent();
            _database = database;
            _repository = new Repository(Settings.Default.ServerName, Settings.Default.Login,
                                         Settings.Default.Password);
            var logins = _repository.GetLoginList();
            foreach (var login in logins)
                LoginListBox.Items.Add(login.Name);
            var roles = _repository.GetRoleList(database);
            foreach (var role in roles)
                RolesListBox.Items.Add(new CheckBox {Content = role});
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            UserTextBox.TextBox.Focus();
        }

        private void OKButton_Click(object sender, RoutedEventArgs e)
        {
            if(LoginListBox.SelectedItem == null)
            {
                MessageBox.Show("Must select a login", "Error creating user", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            try
            {
                var login = LoginListBox.SelectedItem.ToString();
                var roles = new List<string>();
                foreach (CheckBox roleItem in RolesListBox.Items)
                {
                    if(roleItem.IsChecked == true)
                        roles.Add(roleItem.Content.ToString());
                }
                _repository.CreateUser(_database, UserTextBox.Text, login);
                _repository.AddRolesToUser(_database, UserTextBox.Text, roles);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message,
                                "Error creating user", MessageBoxButton.OK,
                                MessageBoxImage.Error);
            }
            Close();
        }

        private void CancelButton_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }
    }
}