﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Collections.Generic;
using System.Windows.Input;
using System.Diagnostics;

using VS2010Addin.SQLAzure2010Addin.ContextMenues;
using VS2010Addin.SQLAzure2010Addin.Data;
using VS2010Addin.SQLAzure2010Addin.Dialogs;
using VS2010Addin.SQLAzure2010Addin.Extensions;
using VS2010Addin.SQLAzure2010Addin.Helpers;
using VS2010Addin.SQLAzure2010Addin.Properties;
using VS2010Addin.SQLAzure2010Addin.Commands;


namespace VS2010Addin.SQLAzure2010Addin.ToolWindows
{
    /// <summary>
    /// Interaction logic for ExplorerControl.xaml
    /// </summary>
    public partial class ExplorerControl
    {

        private bool _loaded;
        private static ExplorerToolWindow _parentWindow;

        public ExplorerControl()
        {
            InitializeComponent();
        }

        public ExplorerControl(ExplorerToolWindow parentWindow)
        {
            InitializeComponent();
            _parentWindow = parentWindow;
        }

        private static IRepository GetRepository()
        {
            return new Repository(Settings.Default.ServerName, Settings.Default.Login, Settings.Default.Password);
        }

        private void MyToolWindow_Loaded(object sender, RoutedEventArgs e)
        {
            if (!_loaded)
            {
                BuildLoginsTree();
                BuildDatabaseTree();
            }

            _loaded = true;
        }

        public void BuildLoginsTree()
        {
            ItemLogins.ContextMenu = new LoginsContextMenu(new LoginCommandParameters { ExplorerControl = this, LoginsTreeViewItem = ItemLogins }, _parentWindow);
            ItemLogins.Items.Clear();
            ItemLogins.Items.Add("Loading...");
            new GetLoginItemsHandler(GetLoginItems).BeginInvoke(ItemLogins, null, null, null);
        }

        private delegate void GetLoginItemsHandler(object sender, RoutedEventArgs args);

        private void GetLoginItems(object sender, RoutedEventArgs args)
        {
            var viewItem = sender as TreeViewItem;
            if (viewItem != null)
            {
                IList<SqlLogin> logins;
                try
                {
                    logins = GetRepository().GetLoginList();
                }
                catch (Exception e)
                {
                    MessageBox.Show("Exception getting logins list: " + e.Message,
                                    "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                Dispatcher.BeginInvoke(new FillLoginItemsHandler(FillLoginItems), viewItem, logins);
            }
            //args.Handled = true;
        }

        private delegate void FillLoginItemsHandler(TreeViewItem parentItem, IList<SqlLogin> childItems);

        private void FillLoginItems(TreeViewItem parentItem, IList<SqlLogin> logins)
        {
            parentItem.Items.Clear();
            foreach (var login in logins)
            {
                var item = TreeViewHelper.CreateTreeViewItemWithImage(login.Name, "../Resources/login.png", false);
                item.ContextMenu = new LoginContextMenu(new LoginCommandParameters
                                                            {
                                                                ExplorerControl = this,
                                                                LoginsTreeViewItem = parentItem,
                                                                Login = login
                                                            }, _parentWindow);

                parentItem.Items.Add(item);
            }
        }

        public void BuildDatabaseTree()
        {
            ItemDatabases.ContextMenu = new DatabasesContextMenu(new DatabasesMenuCommandParameters
                                                                     {
                                                                         ExplorerControl = this,
                                                                         DatabasesTreeViewItem = ItemDatabases
                                                                     }, _parentWindow);
            ItemDatabases.Items.Clear();
            ItemDatabases.Items.Add("Loading...");
            ItemDatabases.IsExpanded = true;
            new GetDatabaseListHandler(GetDatabaseList).BeginInvoke(null, null);
        }

        private delegate void GetDatabaseListHandler();

        private void GetDatabaseList()
        {
            IList<string> databaseList;
            try
            {
                databaseList = GetRepository().GetDatabaseList();
            }
            catch (Exception e)
            {
                MessageBox.Show("Exception getting data from SQL Azure, have you set your credentials? " + e.Message,
                                "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            var fillList = new FillDatabaseListHandler(FillDatabaseList);
            Dispatcher.BeginInvoke(fillList, databaseList); //fill the tree on the UI thread
        }

        private delegate void FillDatabaseListHandler(IList<string> databaseList);

        private void FillDatabaseList(IList<string> databaseList)
        {
            ItemDatabases.Items.Clear();
            foreach (var databaseName in databaseList)
            {
                var databaseTreeViewItem = AddDatabaseToTreeView(databaseName);
                ItemDatabases.Items.Add(databaseTreeViewItem);
            }
            ItemDatabases.IsExpanded = true;
        }

        private TreeViewItem AddDatabaseToTreeView(string database)
        {
            var databaseTreeViewItem = TreeViewHelper.CreateTreeViewItemWithImage(database, "../Resources/database.png", true);
            databaseTreeViewItem.ContextMenu = new DatabaseContextMenu(new DatabaseMenuCommandParameters
                                                                           {
                                                                               ExplorerControl = this,
                                                                               Database = database
                                                                           }, _parentWindow);
            databaseTreeViewItem.Items.Clear();

            var tables = TreeViewHelper.CreateTreeViewItemWithImage("Tables", "../Resources/folder.png", true);
            tables.ContextMenu = new TablesContextMenu(new CreateNewCommandParameters
            {
                Database = database,
                TreeViewItem = tables
            }, _parentWindow);
            tables.Expanded += (sender, args) => new GetTableItemsHandler(GetTableItems).BeginInvoke(sender, args, database, null, null);
            databaseTreeViewItem.Items.Add(tables);

            var views = TreeViewHelper.CreateTreeViewItemWithImage("Views", "../Resources/folder.png", true);
            views.ContextMenu = new ViewsContextMenu(new CreateNewCommandParameters
            {
                Database = database,
                TreeViewItem = views
            }, _parentWindow);
            views.Expanded += (sender, args) => new GetViewItemsHandler(GetViewItems).BeginInvoke(sender, args, database, null, null);
            databaseTreeViewItem.Items.Add(views);

            var sps = TreeViewHelper.CreateTreeViewItemWithImage("Stored Procedures", "../Resources/folder.png", true);
            sps.ContextMenu = new StoredProceduresContextMenu(new CreateNewCommandParameters
            {
                Database = database,
                TreeViewItem = sps
            }, _parentWindow);
            sps.Expanded += (sender, args) => new GetStoredProcsItemsHandler(GetStoredProcsItems).BeginInvoke(sender, args, database, null, null);
            databaseTreeViewItem.Items.Add(sps);

            var functions = TreeViewHelper.CreateTreeViewItemWithImage("Functions", "../Resources/folder.png", true);
            functions.ContextMenu = new FunctionsContextMenu(new CreateNewCommandParameters
            {
                Database = database,
                TreeViewItem = functions
            }, _parentWindow);
            functions.Expanded += (sender, args) => new GetFunctionsItemsHandler(GetFunctionsItems).BeginInvoke(sender, args, database, null, null);
            databaseTreeViewItem.Items.Add(functions);

            var users = TreeViewHelper.CreateTreeViewItemWithImage("Users", "../Resources/folder.png", true);
            users.ContextMenu = new UsersContextMenu(new UsersCommandParameters
                                                         {
                                                             TreeViewItem = users,
                                                             Database = database
                                                         }, this);
            users.Expanded += (sender, args) => new GetUsersItemsHandler(GetUsersItems).BeginInvoke(sender, args, database, null, null);
            databaseTreeViewItem.Items.Add(users);

            return databaseTreeViewItem;
        }

        private delegate void GetTableItemsHandler(object sender, RoutedEventArgs args, string database);

        private void GetTableItems(object sender, RoutedEventArgs args, string database)
        {
            var viewItem = sender as DatabaseTreeViewItem;
            if (viewItem != null)
            {
                IList<string> nameList;
                try
                {
                    nameList = GetRepository().GetTableList(database);
                }
                catch (Exception e)
                {
                    MessageBox.Show("Exception getting table list: " + e.Message,
                                    "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                Dispatcher.BeginInvoke(new FillTableItemsHandler(FillTableItems), database, viewItem, nameList);
            }
            args.Handled = true;
        }

        private delegate void FillTableItemsHandler(string database, DatabaseTreeViewItem parentItem, IList<string> childItems);

        private static void FillTableItems(string database, DatabaseTreeViewItem parentItem, IList<string> nameList)
        {
            parentItem.Items.Clear();
            foreach (var table in nameList)
            {
                var item = TreeViewHelper.CreateTreeViewItemWithImage(table, "../Resources/table.png", true);
                item.ContextMenu = new TableContextMenu(new MenuCommandParameters { Database = database, Name = table, MenuItemType = MenuCommandParameters.MenuType.Table }, _parentWindow);

                item.Expanded += (s, e) => GetTableColumns(s, e, database);
                parentItem.Items.Add(item);
            }
        }

        private static void GetTableColumns(object sender, RoutedEventArgs args, string database)
        {
            var viewItem = sender as DatabaseTreeViewItem;
            if (viewItem != null)
            {
                viewItem.Items.Clear();

                var tableName = viewItem.MetaData.EverythingAfter(".");
                var pkList = GetRepository().GetPrimaryKeys(database, tableName);
                var columns = GetRepository().GetTableOrViewColumns(database, tableName);
                foreach (var column in columns)
                {
                    var i = pkList.Contains(column)
                                ? TreeViewHelper.CreateTreeViewItemWithImage(column,
                                                                             "../Resources/key.png", false)
                                : TreeViewHelper.CreateTreeViewItemWithImage(column,
                                                                             "../Resources/column.png", false);
                    viewItem.Items.Add(i);
                }
            }
            args.Handled = true;
        }

        private delegate void GetViewItemsHandler(object sender, RoutedEventArgs args, string database);

        private void GetViewItems(object sender, RoutedEventArgs args, string database)
        {
            var viewItem = sender as DatabaseTreeViewItem;
            if (viewItem != null)
            {
                IList<string> nameList;
                try
                {
                    nameList = GetRepository().GetViewList(database);
                }
                catch (Exception e)
                {
                    MessageBox.Show("Exception getting view list: " + e.Message,
                                    "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                Dispatcher.BeginInvoke(new FillViewItemsHandler(FillViewItems), database, viewItem, nameList);
            }
            args.Handled = true;
        }

        private delegate void FillViewItemsHandler(string database, DatabaseTreeViewItem parentItem, IList<string> childItems);

        private static void FillViewItems(string database, DatabaseTreeViewItem parentItem, IList<string> nameList)
        {
            parentItem.Items.Clear();
            foreach (var table in nameList)
            {
                var item = TreeViewHelper.CreateTreeViewItemWithImage(table, "../Resources/view.png", true);
                item.ContextMenu = new TableContextMenu(new MenuCommandParameters { Database = database, Name = table, MenuItemType = MenuCommandParameters.MenuType.View }, _parentWindow);

                item.Expanded += (s, e) => GetTableColumns(s, e, database);
                parentItem.Items.Add(item);
            }
        }

        private delegate void GetStoredProcsItemsHandler(object sender, RoutedEventArgs args, string database);

        private void GetStoredProcsItems(object sender, RoutedEventArgs args, string database)
        {
            var viewItem = sender as DatabaseTreeViewItem;
            if (viewItem != null)
            {
                IList<string> nameList;
                try
                {
                    nameList = GetRepository().GetSpList(database);
                }
                catch (Exception e)
                {
                    MessageBox.Show("Exception getting stored procedures list: " + e.Message,
                                    "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                Dispatcher.BeginInvoke(new FillStoredProcsAndFunctionsItemsHandler(FillStoredProcsAndFunctionsItems), database, viewItem, nameList);
            }
            args.Handled = true;
        }

        private delegate void FillStoredProcsAndFunctionsItemsHandler(string database, DatabaseTreeViewItem parentItem, IList<string> childItems);

        private static void FillStoredProcsAndFunctionsItems(string database, DatabaseTreeViewItem parentItem, IList<string> nameList)
        {
            parentItem.Items.Clear();
            foreach (var table in nameList)
            {
                var item = TreeViewHelper.CreateTreeViewItemWithImage(table, "../Resources/sp.png", true);
                item.ContextMenu = new TableContextMenu(new MenuCommandParameters { Database = database, Name = table, MenuItemType = MenuCommandParameters.MenuType.SP }, _parentWindow);

                item.Expanded += (s, e) => GetProcedureParameters(s, e, database);
                parentItem.Items.Add(item);
            }
        }

        private delegate void GetFunctionsItemsHandler(object sender, RoutedEventArgs args, string database);

        private void GetFunctionsItems(object sender, RoutedEventArgs args, string database)
        {
            var viewItem = sender as DatabaseTreeViewItem;
            if (viewItem != null)
            {
                IList<string> nameList;
                try
                {
                    nameList = GetRepository().GetFunctionList(database);
                }
                catch (Exception e)
                {
                    MessageBox.Show("Exception getting functions list: " + e.Message,
                                    "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                Dispatcher.BeginInvoke(new FillStoredProcsAndFunctionsItemsHandler(FillStoredProcsAndFunctionsItems), database, viewItem, nameList);
            }
            args.Handled = true;
        }

        private delegate void GetUsersItemsHandler(object sender, RoutedEventArgs args, string database);

        public void GetUsersItems(object sender, RoutedEventArgs args, string database)
        {
            var viewItem = sender as DatabaseTreeViewItem;
            if (viewItem != null)
            {
                IList<SqlUser> userList;
                try
                {
                    userList = GetRepository().GetUserList(database);
                }
                catch (Exception e)
                {
                    MessageBox.Show("Exception getting user list: " + e.Message,
                                    "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
                Dispatcher.BeginInvoke(new FillUsersItemsHandler(FillUsersItems), database, viewItem, userList);
            }

            if (args != null) args.Handled = true;
        }

        private delegate void FillUsersItemsHandler(string database, DatabaseTreeViewItem parentItem, IList<SqlUser> childItems);

        private void FillUsersItems(string database, DatabaseTreeViewItem parentItem, IList<SqlUser> userList)
        {
            var existList = new List<string>();
            parentItem.Items.Clear();
            foreach (var user in userList)
            {
                if (!existList.Contains(user.Name))
                {
                    var item = TreeViewHelper.CreateTreeViewItemWithImage(user.Name, "../Resources/user.png", false);
                    item.ContextMenu =
                        new UserContextMenu(
                            new UserCommandParameters {TreeViewItem = parentItem, User = user, Database = database},
                            this);
                    parentItem.Items.Add(item);
                    existList.Add(user.Name);
                }
            }
        }

        private static void GetProcedureParameters(object sender, RoutedEventArgs args, string database)
        {
            var viewItem = sender as DatabaseTreeViewItem;
            if (viewItem != null)
            {
                viewItem.Items.Clear();

                var procedure = viewItem.MetaData.EverythingAfter(".");
                var parameters = GetRepository().GetParameters(database, procedure);
                foreach (var parameter in parameters)
                {
                    var item = TreeViewHelper.CreateTreeViewItemWithImage(parameter, "../Resources/column.png", false);
                    viewItem.Items.Add(item);
                }
            }
            args.Handled = true;
        }

        private void TreeViewItem_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            ((TreeViewItem)sender).IsSelected = true;
            e.Handled = true;
        }

        private void ToolbarProperties_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new PropertiesDialog();
            dialog.ShowDialog();

            BuildLoginsTree();
            BuildDatabaseTree();
        }

        private void ToolbarResources_Click(object sender, RoutedEventArgs e)
        {
            Process.Start("http://buzzfrog.blogs.com/zabrak/sql-azure-resources.html");
        }

        private void ToolbarRefresh_Click(object sender, RoutedEventArgs e)
        {
            BuildLoginsTree();
            BuildDatabaseTree();
        }

        private void ToolbarAbout_Click(object sender, RoutedEventArgs e)
        {
            var dialog = new AboutDialog();
            dialog.ShowDialog();
        }
    }
}