﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;
using System.Text;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;

using VS2010Addin.SQLAzure2010Addin.Data;
using VS2010Addin.SQLAzure2010Addin.Properties;
using VS2010Addin.SQLAzure2010Addin.ScriptParsing;

namespace VS2010Addin.SQLAzure2010Addin.ToolWindows
{
    /// <summary>
    /// Interaction logic for TableDataControl.xaml
    /// </summary>
    public partial class SqlEditorControl
    {
        public string Database { get; set; } //This property must be set by parent window

        public SqlEditorControl()
        {
            InitializeComponent();
        }

        #region Toolbar Button events

        private void SqlEditorWindow_Loaded(object sender, RoutedEventArgs e)
        {
            SqlTextBox.Focus();
        }

        private void ExecuteButton_Click(object sender, RoutedEventArgs e)
        {
            ExecuteSqlScriptInEditor();
        }

        private void ParseButton_Click(object sender, RoutedEventArgs e)
        {
            var parser = new Parser();
            try
            {
                parser.Parse(SqlTextBox.Text);
                ResultsTextBox.Text = "Script seems alright!";
            }
            catch (Exception ex)
            {
                ResultsTextBox.Text = ex.Message;
            }
        }

        private void FormatButton_Click(object sender, RoutedEventArgs e)
        {
            var parser = new Parser();
            try
            {
                SqlTextBox.Text = parser.Parse(SqlTextBox.Text);
            }
            catch (Exception ex)
            {
                ResultsTextBox.Text = ex.Message;
            }
        }


        #endregion

        private void ExecuteSqlScriptInEditor()
        {
            Debug.Assert(!string.IsNullOrEmpty(Database), "Database property of this control has not been set by parent window or control");

            var repository = new Repository(Settings.Default.ServerName, Settings.Default.Login,
                                            Settings.Default.Password);
            try
            {
                var sql = GetSqlFromSqlEditorTextBox();
                if (sql.Length == 0) return;

                ResultsTextBox.Text = "";
                var dataset = repository.ExecuteSql(sql, Database);
                if (dataset != null)
                {
                    ParseDataSetResultsToResultsBox(dataset);
                }
            }
            catch (SqlException sqlException)
            {
                ParseSqlErrorToResultsBox(sqlException);
            }
        }

        private string GetSqlFromSqlEditorTextBox()
        {
            var sql = SqlTextBox.Text;
            if (!string.IsNullOrEmpty(SqlTextBox.SelectedText))
                sql = SqlTextBox.SelectedText;
            return sql;
        }

        private void ParseSqlErrorToResultsBox(SqlException sqlException)
        {
            ResultsTextBox.Foreground = Brushes.Red;
            ResultsTextBox.Text = sqlException.Message;
        }

        private void ParseDataSetResultsToResultsBox(DataSet dataset)
        {
            ResultsTextBox.Foreground = Brushes.Black; 
            var results = new StringBuilder();
            
            if(dataset.Tables.Count == 0)
            {
                ResultsTextBox.Text = "Command(s) completed successfully.";
                return;
            }

            foreach (DataTable table in dataset.Tables)
            {
                foreach (var column in table.Columns)
                {
                    results.Append(column + "\t");
                }
                results.AppendLine("\n--------------------------------------------------------------------------------------");

                foreach (DataRow row in table.Rows)
                {
                    foreach (var item in row.ItemArray)
                        results.Append(item + "\t");
                    results.Append("\n");
                }
                results.AppendLine("\n");
            }

           
            ResultsTextBox.Text = results.ToString();
        }

      
        #region Hotkey events and management

        private bool _controlPressed;
        private bool _shiftPressed;

        private void SqlTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (_controlPressed)
            {
                switch (e.Key)
                {
                    case Key.E:
                        ExecuteSqlScriptInEditor();
                        _controlPressed = false;
                        e.Handled = true;
                        break;
                }
            }
            else
            {
                switch (e.Key)
                {
                    case Key.LeftShift:
                    case Key.RightShift:
                        _shiftPressed = true;
                        return;
                    case Key.LeftCtrl:
                    case Key.RightCtrl:
                        _controlPressed = true;
                        return;
                }

            }
        }

        private void SqlTextBox_KeyUp(object sender, KeyEventArgs e)
        {
            switch (e.Key)
            {
                case Key.RightCtrl:
                case Key.LeftCtrl:
                    _controlPressed = false;
                    break;
                case Key.RightShift:
                case Key.LeftShift:
                    _shiftPressed = false;
                    break;
            }
        }

        private void SqlTextBox_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            if (_shiftPressed && e.Key != Key.RightShift && e.Key != Key.LeftShift)
            {
                switch (e.Key)
                {
                    case Key.Enter:
                        ExecuteSqlScriptInEditor();
                        _shiftPressed = false;
                        e.Handled = true;
                        break;
                }
            }
        }

        #endregion

    }
}