﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;

using Microsoft.Data.Schema.ScriptDom;
using Microsoft.Data.Schema.ScriptDom.Sql;

namespace VS2010Addin.SQLAzure2010Addin.ScriptParsing
{
    public class Parser
    {
        public string Parse(string sql)
        {
            if (sql == null) return "";

            var parser = new TSql100Parser(false);
            var options = new SqlScriptGeneratorOptions
                              {
                                  SqlVersion = SqlVersion.Sql100,
                                  KeywordCasing = KeywordCasing.Uppercase
                              };
            var scriptGen = new Sql100ScriptGenerator(options);
            using (var sr = new StringReader(sql))
            {
                IList<ParseError> errors;
                var fragment = parser.Parse(sr, out errors);
                if (errors != null && errors.Count > 0)
                {
                    throw new ApplicationException(BuildErrorMessage(errors));
                }

                var script = fragment as TSqlScript;
                if (script != null)
                {
                    string pretty;
                    scriptGen.GenerateScript(fragment, out pretty);
                    return pretty;
                }
            }
            return "";
        }

        private static String BuildErrorMessage(IEnumerable<ParseError> errors)
        {
            var sb = new StringBuilder();
            foreach (var error in errors)
            {
                sb.Append(error.Message);
                sb.Append(" offset " + error.Offset);
                sb.Append(" row: " + error.Line);
                sb.Append(" column: " + error.Column);
            }
            return sb.ToString();
        }
    }
}
